/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer.data;

import java.beans.IntrospectionException;
import nl.cloudfarming.client.geoviewer.Category;
import java.util.Set;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.MapController;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;

/**
 * Used to initialize the nodes for the layers.
 * 
 * @author Timon Veenstra
 */
public class LayerChildren extends Children.Keys<Layer> {


    public LayerChildren() {
    }

    /**
     * Initializes the layers (Nodes)
     */
    @Override
    @SuppressWarnings("unchecked")
    protected void addNotify() {
        super.addNotify();
        Set<Layer> layers = LayerDataManager.getInstance().getLayersForCategory(getNode().getLookup().lookup(Category.class));
        if (layers != null && layers.size() > 0) {
            setKeys(layers);
        }
    }

    @Override
    protected Node[] createNodes(Layer key) {
        try {
            return new Node[]{new LayerNode(key,getNode().getLookup().lookup(MapController.class))};
        } catch (IntrospectionException ex) {
            Exceptions.printStackTrace(ex);
            return null;
        }
    }
}
