/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer.data;

import nl.cloudfarming.client.geoviewer.Category;
import nl.cloudfarming.client.geoviewer.LayerObject;
import org.openide.nodes.Children;
import org.openide.nodes.Node;

/**
 *
 * Node utility class
 *
 * @author Timon Veenstra
 */
public class NodeUtils {

    /**
     * Find the node for a layer object in the ExplorerDataProvider
     *
     * @param layerObject
     * @return
     */
    static LayerObjectNode findNodeForObject(LayerObject layerObject) {
        CategoryChildren categoryChildren = findCategoryChildren();
        for (Node categoryNode: categoryChildren.getNodes()){
             for (Node layerNode: categoryNode.getChildren().getNodes()){
                for (Node layerObjectNode: layerNode.getChildren().getNodes()){
                    if (layerObject.equals(layerObjectNode.getLookup().lookup(LayerObject.class))){
                        return (LayerObjectNode)layerObjectNode;
                    }
                }
             }
        }
        return null;
    }

    /**
     * find the layerchilden for a category in the ExplorerDataProvider
     * will throw a InvalidNodeHierarchyException when the layerChildren are not found
     *
     * @param manager
     * @param category
     * @return
     */
    static LayerChildren findLayerChildrenForCategory(
            Category category) {
        CategoryNode categoryNode = findCategoryNode(category);
        if (categoryNode.getChildren() instanceof LayerChildren) {
            return (LayerChildren) categoryNode.getChildren();
        }
        throw new InvalidNodeHierarchyException();
    }

    /**
     * find category children in the ExplorerDataProvider
     *
     * @return CategoryChildren
     */
    static CategoryChildren findCategoryChildren() {
        Children categoryChildren = findCategoryChildren(LayerDataManager.getInstance().getExplorerDataProvider().getExplorerManager().getRootContext());
        if (categoryChildren instanceof CategoryChildren) {
            return (CategoryChildren) categoryChildren;
        }
        throw new InvalidNodeHierarchyException();
    }


    /**
     * search recursivly for CategoryChildren
     * 
     * @param node
     * @return
     */
    private static CategoryChildren findCategoryChildren(Node node){
        if (node.getChildren() == null){
            return null;
        }
        if (node.getChildren() instanceof CategoryChildren){
            return (CategoryChildren) node.getChildren();
        }else{
            for (Node child: node.getChildren().getNodes()){
                CategoryChildren categoryChildren = findCategoryChildren(child);
                if (categoryChildren != null){
                    return categoryChildren;
                }
            }
        }
        return null;
    }


    /**
     * 
     * @param manager
     * @param category
     * @return
     */
    static CategoryNode findCategoryNode(Category category) {
        Node foundNode = findCategoryChildren().findChild(category.getName());
        if (foundNode instanceof CategoryNode) {
            return (CategoryNode) foundNode;
        }
        throw new InvalidNodeHierarchyException();
    }

    public static class InvalidNodeHierarchyException extends RuntimeException {

        public InvalidNodeHierarchyException(Throwable cause) {
            super(cause);
        }

        public InvalidNodeHierarchyException(String message, Throwable cause) {
            super(message, cause);
        }

        public InvalidNodeHierarchyException(String message) {
            super(message);
        }

        public InvalidNodeHierarchyException() {
        }
    }
}
